#include <stdio.h>
#include <error.h>

#define STACK_SIZE 10

int stack[STACK_SIZE];
int stack_pointer = 0;

void push (int x)
{
  if (stack_pointer >= STACK_SIZE)
    error (1, 0, "push(): stack overflow");
  stack[stack_pointer++] = x;
}

int pop (void)
{
  if (stack_pointer == 0)
    error (1, 0, "pop(): stack is empty");
  return stack[--stack_pointer];
}

void show (void)
{
  printf ("stack content:");
  for (int i = 0; i < stack_pointer; i++)
    printf (" %d", stack[i]);
  if (stack_pointer)
    printf ("\n");
  else
    printf (" (empty)\n");
}

void insert (int x, int pos)
{
  if (pos < 0 || pos >= STACK_SIZE)
    error (1, 0, "insert(): position %d out of range (0, ..., %d)",
                 pos, STACK_SIZE - 1);
  for (int i = stack_pointer - 1; i >= pos; i--)
    stack[i + 1] = stack[i];
  stack[pos] = x;
  stack_pointer++;
}

void insert_sorted (int x)
{
  if (stack_pointer >= STACK_SIZE)
    error (1, 0, "insert_sorted(): stack overflow");
  int i = stack_pointer - 1;
  while (i >= 0 && x < stack[i])
    {
      stack[i + 1] = stack[i];
      i--;
    }
  stack[i + 1] = x;
  stack_pointer++;
}

int search (int x)
{
  int left = 0;
  int right = stack_pointer;
  while (left < right - 1)
    {
      int middle = (left + right) / 2;
      if (x < stack[middle])
        right = middle;
      else
        left = middle;
    }
  if (x == stack[left])
    return left;
  else
   return -1;
}

int main (void)
{
  push (3);
  push (7);
  push (137);
  show ();
  insert (5, 1);
  show ();
  insert_sorted (42);
  show ();
  insert_sorted (2);
  show ();
  printf ("%d\n", search (2));
  printf ("%d\n", search (4));
  printf ("%d\n", search (42));
  printf ("%d\n", search (67));
  printf ("%d\n", search (137));
  printf ("%d\n", search (1117));
//  insert (512, -1);
//  insert (512, 42);
//  for (int i = 255; i > 128; i--)
//    push (i);
  for (int i = 255; i > 128; i--)
    insert_sorted (i);
//  for (int i = 0; i < 1000; i++)
//    printf ("%d\n", pop ());
  return 0;
}
